package pro.litvinovg.w2phtml.gui;

import static pro.litvinovg.w2phtml.gui.Contstants.*;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.HashMap;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

public class BasePanel extends JPanel {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private Localizer localizer = null;
	private JFrame singleFrame = null;
	private UIPreferences preferences = null;
	private HashMap<String, Component> configuration = null;
	public BasePanel(Localizer localizer, JFrame singleFrame, HashMap<String, Component> configuration, UIPreferences preferences) {
		this.localizer = localizer;
		this.singleFrame = singleFrame;
		this.configuration = configuration;
		this.preferences = preferences;
	}
	
	public BasePanel(Localizer localizer, JFrame singleFrame) {
		this.localizer = localizer;
		this.singleFrame = singleFrame;
	}
	
	protected JCheckBox addMathJaxCB(Localizer localizer, UIPreferences preferences) {
		JCheckBox cb_UseMathJax = new JCheckBox(localizer.getTranslation(LABEL_USE_MATHJAX));
		cb_UseMathJax.setSelected(Boolean.parseBoolean(preferences.get(PREF_USE_MATHJAX)));
		return cb_UseMathJax;
	}
	
	protected void setOutputFilePath(String newFilePath, JTextField tf_OutputFile, String extension) {

		File inputFile = new File(newFilePath);
		if (!inputFile.exists()) {
			return;
		}
		if (!inputFile.isDirectory()) {
			File parent = inputFile.getParentFile();
			if (parent == null) {
				return;
			}
			if (!parent.canWrite()) {
				JOptionPane.showMessageDialog(singleFrame, localizer.getTranslation(CANT_WRITE_MESSAGE));
			}
		}
		if (newFilePath.length() < 3) {
			return;
		}
		String exportPath;
		if (inputFile.isDirectory()) {
			exportPath = newFilePath;
			tf_OutputFile.setText(exportPath);
		} else {
			String importExt = newFilePath.substring(newFilePath.length() - 3, newFilePath.length()).toLowerCase();
			if (importExt.equals(ODT_FILE_EXTENSION)) {
				exportPath = newFilePath.substring(0, newFilePath.length() - 3) + extension;
				tf_OutputFile.setText(exportPath);
			}
		}
	}
	
	protected JCheckBox addIgnoreEmptyParsCB() {
		JCheckBox cb_IgnoreEmptyParagraphs = new JCheckBox(localizer.getTranslation(LABEL_INGNORE_EMPTY_PARS));
		cb_IgnoreEmptyParagraphs.setSelected(Boolean.parseBoolean(preferences.get(PREF_IGNORE_EMPTY_PARS)));
		configuration.put(ARG_IGNORE_EMPTY_PARAGRAPHS, cb_IgnoreEmptyParagraphs);
		return cb_IgnoreEmptyParagraphs;
	}

	protected JCheckBox addSplitWholePagesOnly() {
		JCheckBox cb_SplitWholePagesOnly = new JCheckBox(localizer.getTranslation(LABEL_SPLIT_WHOLE_PAGES_ONLY));
		cb_SplitWholePagesOnly.setSelected(Boolean.parseBoolean(preferences.get(PREF_SPLIT_WHOLE_PAGES)));
		configuration.put(ARG_ALIGN_SPLITS_TO_PAGES, cb_SplitWholePagesOnly);
		return cb_SplitWholePagesOnly;
	}
	
	protected JCheckBox addMathJaxCB() {
		JCheckBox cb_UseMathJax = new JCheckBox(localizer.getTranslation(LABEL_USE_MATHJAX));
		cb_UseMathJax.setSelected(Boolean.parseBoolean(preferences.get(PREF_USE_MATHJAX)));
		configuration.put(ARG_USE_MATHJAX, cb_UseMathJax);
		return cb_UseMathJax;
	}
	protected JCheckBox addIgnoreHardLineBreaks() {
		JCheckBox cb_IgnoreHardLineBreaks = new JCheckBox(localizer.getTranslation(LABEL_FILETER_HARD_LINE_BREAKS));
		cb_IgnoreHardLineBreaks.setSelected(Boolean.parseBoolean(preferences.get(PREF_IGNORE_HARD_BREAKS)));
		configuration.put(ARG_IGNORE_HARD_LINE_BREAKS, cb_IgnoreHardLineBreaks);
		return cb_IgnoreHardLineBreaks;
	}

	protected JTextField addLetterSpacingFilter() {
		JTextField tf_FilterLetterSpacing = new JTextField();
		tf_FilterLetterSpacing.setText(preferences.get(PREF_LETTER_SPACING));
		configuration.put(ARG_MIN_LETTER_SPACING, tf_FilterLetterSpacing);
		tf_FilterLetterSpacing.setColumns(10);
		return tf_FilterLetterSpacing;
	}

	protected JTextField addSplitByLevel() {
		JTextField tf_SplitByLevel = new JTextField();
		configuration.put(ARG_SPLIT_LEVEL, tf_SplitByLevel);
		tf_SplitByLevel.setText(preferences.get(PREF_SPLIT_BY_LEVEL));
		tf_SplitByLevel.setColumns(10);
		return tf_SplitByLevel;
	}

	protected JCheckBox addInlineStyles() {
		JCheckBox cb_InlineStyles = new JCheckBox(localizer.getTranslation(LABEL_INLINE_STYLES));
		cb_InlineStyles.setSelected(Boolean.parseBoolean(preferences.get(PREF_INLINE_STYLES)));
		configuration.put(ARG_CSS_INLINE, cb_InlineStyles);
		return cb_InlineStyles;
	}
	
	protected JCheckBox addGreenstoneTags() {
		JCheckBox cb_Greenstone = new JCheckBox(localizer.getTranslation(LABEL_GREENSTONE_SPLIT));
		configuration.put(ARG_GREENSTONE, cb_Greenstone);
		cb_Greenstone.setSelected(Boolean.parseBoolean(preferences.get(PREF_GREENSTONE_TAGS)));
		return cb_Greenstone;
	}

	protected JTextField addOutputFileTextField() {
		JTextField tf_OutputFile = new JTextField("");
		configuration.put(ARG_OUTPUT_FILE, tf_OutputFile);
		tf_OutputFile.setColumns(10);
		return tf_OutputFile;
	}

	protected JCheckBox addPagination() {
		JCheckBox cb_Pagination = new JCheckBox(localizer.getTranslation(LABEL_PAGINATION));
		cb_Pagination.setSelected(Boolean.getBoolean(preferences.get(PREF_PAGINATION)));
		configuration.put(ARG_PAGINATION, cb_Pagination);
		return cb_Pagination;
	}
	
	protected JCheckBox addConvertToPX() {
		JCheckBox cb_convertToPx = new JCheckBox(localizer.getTranslation(LABEL_CONVERT_TO_PX));
		cb_convertToPx.setSelected(Boolean.getBoolean(preferences.get(PREF_CONVERT_TO_PX)));
		configuration.put(ARG_CONVERT_TO_PX, cb_convertToPx);
		return cb_convertToPx;
	}

	protected JTextField addPageBreakStyle() {
		JTextField tf_pageBreakInlineStyle = new JTextField(preferences.get(PREF_PAGEBREAK_STYLES));
		tf_pageBreakInlineStyle.setColumns(10);
		configuration.put(ARG_PAGE_BREAK_STYLE, tf_pageBreakInlineStyle);
		return tf_pageBreakInlineStyle;
	}

	protected JLabel addTargetFormat(String format) {
		JLabel lb_TargetFormat = new JLabel(format);
		configuration.put(ARG_TARGET_FORMAT, lb_TargetFormat);
		return lb_TargetFormat;
	}

	protected JTextField addScaling() {
		JTextField tf_Scale = new JTextField();
		tf_Scale.setText(preferences.get(PREF_SCALING));
		tf_Scale.setColumns(10);
		configuration.put(ARG_SCALING, tf_Scale);
		return tf_Scale;
	}
	
	protected JTextField addInputFile() {
		JTextField tf_inputFile = new JTextField(preferences.getSourceFileName());
		tf_inputFile.setColumns(10);
		configuration.put(ARG_INPUT_FILE, tf_inputFile);
		return tf_inputFile;
	}

	protected JComboBox addImageResolution() {
		JComboBox cbox_image_resolution = new JComboBox(preferences.getAll(PREF_RESOLUTIONS));
		cbox_image_resolution.setSelectedIndex(Integer.parseInt(preferences.get(PREF_DEFAULT_IMAGE_RESOLUTION)));
		configuration.put(ARG_IMAGE_RESOLUTION, cbox_image_resolution);
		return cbox_image_resolution;
	}
	
	protected JTextField addMaxWidth() {
		JTextField tf_MaxWidth = new JTextField(preferences.get(PREF_MAX_WIDTH));
		tf_MaxWidth.setColumns(10);
		configuration.put(ARG_MAX_WIDTH, tf_MaxWidth);
		return tf_MaxWidth;
	}
	
	protected JCheckBox addEmbedRasterImages() {
		JCheckBox cb_EmbedRaster = new JCheckBox(localizer.getTranslation(LABEL_EMBED_RASTER));
		cb_EmbedRaster.setSelected(Boolean.parseBoolean(preferences.get(PREF_EMBED_RASTER)));
		configuration.put(ARG_EMBED_IMG, cb_EmbedRaster);
		return cb_EmbedRaster;
	}

	protected JCheckBox addEmbedVectorImages() {
		JCheckBox cb_EmbedVectorImages = new JCheckBox(localizer.getTranslation(LABEL_EMBED_VECTOR));
		cb_EmbedVectorImages.setSelected(Boolean.parseBoolean(preferences.get(PREF_EMBED_VECTOR)));
		configuration.put(ARG_EMBED_SVG, cb_EmbedVectorImages);
		return cb_EmbedVectorImages;
	}
	
	protected JButton addChooseOutputButton(JTextField tf_OutputFile, String fileExtension) {
		JButton btn_ChooseOutputFile = new JButton(localizer.getTranslation(LABEL_BUTTON_CHOOSE));
		btn_ChooseOutputFile.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				FileDialog fileDialog = new FileDialog();
				String newFilePath = fileDialog.chooseFile(tf_OutputFile.getText(),fileExtension);
				if (newFilePath != null && !newFilePath.isEmpty()) {
					tf_OutputFile.setText(newFilePath);
				}
			}
		});
		return btn_ChooseOutputFile;
	}
	
	protected void setMetadataFilePath(String newFilePath, JTextField tf_MetadataFile, String extension) {
		
		File inputFile = new File(newFilePath);
		if (!inputFile.exists()) {
			return;
		}
		if (inputFile.isDirectory()) {
			tf_MetadataFile.setText(newFilePath);	
		} else {
			File parent = inputFile.getParentFile();
			if (parent == null) {
				return;
			}
			if (newFilePath.length() < 3) {
				return;
			}
			String importExt = newFilePath.substring(newFilePath.length()-3, newFilePath.length()).toLowerCase();
			if (importExt.equals(ODT_FILE_EXTENSION)) {
				String metadataPath = newFilePath.substring(0, newFilePath.length()-3) + extension;
				File metaFile = new File(metadataPath);
				if (metaFile.exists() && metaFile.canRead()){
					tf_MetadataFile.setText(metadataPath);	
				}	else {
					tf_MetadataFile.setText("");
				}
			}
		}
		
	}
}